@@ -73,19 +73,21 @@ def getfile(filename, mode):
 def _readline(fd):
     line = fd.readline()
     if isinstance(line, bytes):
-        return line.decode('UTF-8', 'replace')
+        try:
+            return line.decode('UTF-8')
+        except UnicodeDecodeError:
+            print(f'Warning: non-UTF-8 decodable log line in {fd.name}: {line!r}', file=sys.stderr)
+            return line.decode('UTF-8', 'replace')
     return line
 
 
 def main(files, ignore_exceptions=[], out=sys.stdout):
     tracebacks = {}
     for file_ in files:
-        with getfile(file_, 'rt') as (fd, filename):
+        with getfile(file_, 'rb') as (fd, filename):
             line = True
             while line:
                 line = _readline(fd)
-                if isinstance(line, bytes):
-                    line = line.decode('UTF-8', 'replace')
                 if line.endswith('Traceback (most recent call last):\n'):
                     # TODO: refactor: can probably be done easily by just fetching 2 lines until first line doesn't start with '.*File "' and strip the start
                     # please note that tracebacks may leave out the source code of lines in some certain situations
@@ -217,8 +219,9 @@ COMMON_EXCEPTIONS = (
     E("FileNotFoundError: \\[Errno 2\\] No such file or directory: '/etc/machine.secret'", ['univention/lib/umc.py.*in authenticate_with_machine_account'], 53670),
     E(r"TypeError: modify\(\) got an unexpected keyword argument 'rename_callback'", ['_register_app'], 54578),
     E('sqlite3.OperationalError: no such table: S4 rejected', 'stdin', 54586),
-    # during UCS 5.0-0-errata updates
+    # during UCS 5.0-x-errata updates
     E(r"TypeError: __init__\(\) got an unexpected keyword argument 'cli_enabled'", ['_register_app'], 54584),
+    E(r"FileNotFoundError: \[Errno 2\] No such file or directory: '/usr/share/univention-management-console/oidc/oidc.json'", ['server.py'], 49006),
 
     # updater test cases:
     E('urllib.error.URLError: .*', ['updater/tools.py.*in access']),
@@ -251,7 +254,7 @@ COMMON_EXCEPTIONS = (
     E(r'gaierror: \[Errno -5\] No address associated with hostname'),
     E('.*moduleCreationFailed: Target directory.*not below.*'),
     E("univention.udm.exceptions.NoObject: No object found at DN 'cn=internal-name-for-(folder|category|entry)", ['in refresh'], 53333),  # 86_selenium/185_portal_administration_inline_creation  # Bug #53333
-    E("univention.admin.uexceptions.noObject: cn=internal-name-for-(folder|category|entry),cn=entry,cn=portals", None, 53333),  # 86_selenium/185_portal_administration_inline_creation  # Bug #53333
+    E("univention.admin.uexceptions.noObject:.*cn=internal-name-for-(folder|category|entry),cn=entry,cn=portals", None, 53333),  # 86_selenium/185_portal_administration_inline_creation  # Bug #53333
     E("ldap.NO_SUCH_OBJECT:.*'matched': 'cn=entry,cn=portals,cn=univention,"),  # 86_selenium/185_portal_administration_inline_creation  # Bug #53333
     E('univention.testing.utils.LDAPObjectNotFound: DN:', ['test_container_cn_rename_uppercase_rollback_with_special_characters'], 53776),
     E('dns.resolver.NoAnswer: The DNS response does not contain an answer to the question:', ['test__dns_reverse_zone_check_resolve', 'test_dns_reverse_zone_check_resolve'], 53775),
@@ -351,7 +354,7 @@ COMMON_EXCEPTIONS = (
     E('INSUFFICIENT_ACCESS:.*', ['in password_sync_s4_to_ucs'], 53721),
     E("ModuleNotFoundError: No module named 'univention.config_registry'", ['/usr/sbin/univention-config-registry'], 53765),
     E("AttributeError: module 'univention.admin.syntax' has no attribute 'UMCMessageCatalogFilename_and_GNUMessageCatalog'", ['_unregister_app', 'import_hook_files', 'pupilgroups.py'], 53754),
-    E("AttributeError: module 'univention.admin.syntax' has no attribute 'emailAddressThatMayEndWithADot'", ['_update_modules', '/usr/sbin/univention-management-console-server'], 55590),
+    E("AttributeError: module 'univention.admin.syntax' has no attribute 'emailAddressThatMayEndWithADot'", ['_update_modules', '/usr/sbin/univention-management-console-server', 'forward_zone.py'], 55590),
     E('univention.admin.uexceptions.noObject: uid=.*', ['connector/ad/.*set_userPrincipalName_from_ucr'], 53769),
     E('ldap.TYPE_OR_VALUE_EXISTS:.*SINGLE-VALUE attribute description.*specified more than once', ['sync_from_ucs'], 52801),
     E('univention.admin.uexceptions.wrongObjectType: .*relativeDomainName=.* is not recognized as dns/txt_record.', ['ucs_txt_record_create'], 53425),
@@ -421,7 +424,7 @@ if __name__ == '__main__':
     parser = argparse.ArgumentParser(description=__doc__)
     parser.add_argument('--ignore-exception', '-i', action='append', type=E, default=[E('^$')])
     parser.add_argument('-d', '--default-exceptions', action='store_true')
-    parser.add_argument('files', type=argparse.FileType('r'), nargs='+')
+    parser.add_argument('files', type=argparse.FileType('rb'), nargs='+')
     args = parser.parse_args()
     ignore_exceptions = COMMON_EXCEPTIONS if args.default_exceptions else args.ignore_exception
     sys.exit(int(not main(args.files, ignore_exceptions=ignore_exceptions)))
